@@ -1,12 +1,13 @@
 import dataclasses
+import platform
 import re
 import sys
 import typing
-from typing import Any, Optional, Tuple
+from typing import Any, Optional
 
 import pytest
 
-from pydantic import BaseModel, PydanticUserError, ValidationError
+from pydantic import BaseModel, PydanticUserError, TypeAdapter, ValidationError
 
 
 def test_postponed_annotations(create_module):
@@ -57,16 +58,14 @@ def test_forward_ref_auto_update_no_model(create_module):
             b: 'Foo'
 
     assert module.Bar.__pydantic_complete__ is True
-    assert repr(module.Bar.model_fields['b']) == 'FieldInfo(annotation=Foo, required=True)'
+    assert module.Bar.model_fields['b']._complete
 
     # Bar should be complete and ready to use
     b = module.Bar(b={'a': {'b': {}}})
     assert b.model_dump() == {'b': {'a': {'b': {'a': None}}}}
 
-    # model_fields is complete on Foo
-    assert repr(module.Foo.model_fields['a']) == (
-        'FieldInfo(annotation=Union[Bar, NoneType], required=False, default=None)'
-    )
+    # model_fields is *not* complete on Foo
+    assert not module.Foo.model_fields['a']._complete
 
     assert module.Foo.__pydantic_complete__ is False
     # Foo gets auto-rebuilt during the first attempt at validation
@@ -129,15 +128,13 @@ def test_self_forward_ref_module(create_module):
 def test_self_forward_ref_collection(create_module):
     @create_module
     def module():
-        from typing import Dict, List
-
         from pydantic import BaseModel
 
         class Foo(BaseModel):
             a: int = 123
             b: 'Foo' = None
-            c: 'List[Foo]' = []
-            d: 'Dict[str, Foo]' = {}
+            c: 'list[Foo]' = []
+            d: 'dict[str, Foo]' = {}
 
     assert module.Foo().model_dump() == {'a': 123, 'b': None, 'c': [], 'd': {}}
     assert module.Foo(b={'a': '321'}, c=[{'a': 234}], d={'bar': {'a': 345}}).model_dump() == {
@@ -164,8 +161,8 @@ def test_self_forward_ref_collection(create_module):
     assert repr(module.Foo.model_fields['b']) == 'FieldInfo(annotation=Foo, required=False, default=None)'
     if sys.version_info < (3, 10):
         return
-    assert repr(module.Foo.model_fields['c']) == ('FieldInfo(annotation=List[Foo], required=False, ' 'default=[])')
-    assert repr(module.Foo.model_fields['d']) == ('FieldInfo(annotation=Dict[str, Foo], required=False, default={})')
+    assert repr(module.Foo.model_fields['c']) == ('FieldInfo(annotation=list[Foo], required=False, default=[])')
+    assert repr(module.Foo.model_fields['d']) == ('FieldInfo(annotation=dict[str, Foo], required=False, default={})')
 
 
 def test_self_forward_ref_local(create_module):
@@ -234,14 +231,14 @@ def test_forward_ref_sub_types(create_module):
 def test_forward_ref_nested_sub_types(create_module):
     @create_module
     def module():
-        from typing import ForwardRef, Tuple, Union
+        from typing import ForwardRef, Union
 
         from pydantic import BaseModel
 
         class Leaf(BaseModel):
             a: str
 
-        TreeType = Union[Union[Tuple[ForwardRef('Node'), str], int], Leaf]
+        TreeType = Union[Union[tuple[ForwardRef('Node'), str], int], Leaf]
 
         class Node(BaseModel):
             value: int
@@ -264,13 +261,11 @@ def test_forward_ref_nested_sub_types(create_module):
 def test_self_reference_json_schema(create_module):
     @create_module
     def module():
-        from typing import List
-
         from pydantic import BaseModel
 
         class Account(BaseModel):
             name: str
-            subaccounts: List['Account'] = []
+            subaccounts: list['Account'] = []
 
     Account = module.Account
     assert Account.model_json_schema() == {
@@ -331,8 +326,6 @@ class Account(BaseModel):
 def test_circular_reference_json_schema(create_module):
     @create_module
     def module():
-        from typing import List
-
         from pydantic import BaseModel
 
         class Owner(BaseModel):
@@ -341,7 +334,7 @@ def test_circular_reference_json_schema(create_module):
         class Account(BaseModel):
             name: str
             owner: 'Owner'
-            subaccounts: List['Account'] = []
+            subaccounts: list['Account'] = []
 
     Account = module.Account
     assert Account.model_json_schema() == {
@@ -422,7 +415,7 @@ def test_forward_ref_with_field(create_module):
     @create_module
     def module():
         import re
-        from typing import ForwardRef, List
+        from typing import ForwardRef
 
         import pytest
 
@@ -431,7 +424,7 @@ def test_forward_ref_with_field(create_module):
         Foo = ForwardRef('Foo')
 
         class Foo(BaseModel):
-            c: List[Foo] = Field(..., gt=0)
+            c: list[Foo] = Field(gt=0)
 
         with pytest.raises(TypeError, match=re.escape("Unable to apply constraint 'gt' to supplied value []")):
             Foo(c=[Foo(c=[])])
@@ -446,7 +439,7 @@ from pydantic import BaseModel, Field
 
 
 class Spec(BaseModel):
-    spec_fields: list[str] = Field(..., alias="fields")
+    spec_fields: list[str] = Field(alias="fields")
     filter: str | None = None
     sort: str | None
 
@@ -490,7 +483,7 @@ from __future__ import annotations
 from dataclasses import dataclass
 
 from pydantic import BaseModel
-from typing_extensions import Literal
+from typing import Literal
 
 @dataclass
 class Base:
@@ -507,7 +500,7 @@ class What(BaseModel):
 
 def test_nested_forward_ref():
     class NestedTuple(BaseModel):
-        x: Tuple[int, Optional['NestedTuple']]
+        x: tuple[int, Optional['NestedTuple']]
 
     obj = NestedTuple.model_validate({'x': ('1', {'x': ('2', {'x': ('3', None)})})})
     assert obj.model_dump() == {'x': (1, {'x': (2, {'x': (3, None)})})}
@@ -516,9 +509,7 @@ def test_nested_forward_ref():
 def test_discriminated_union_forward_ref(create_module):
     @create_module
     def module():
-        from typing import Union
-
-        from typing_extensions import Literal
+        from typing import Literal, Union
 
         from pydantic import BaseModel, Field
 
@@ -558,13 +549,13 @@ def test_discriminated_union_forward_ref(create_module):
             'Cat': {
                 'title': 'Cat',
                 'type': 'object',
-                'properties': {'type': {'const': 'cat', 'enum': ['cat'], 'title': 'Type', 'type': 'string'}},
+                'properties': {'type': {'const': 'cat', 'title': 'Type', 'type': 'string'}},
                 'required': ['type'],
             },
             'Dog': {
                 'title': 'Dog',
                 'type': 'object',
-                'properties': {'type': {'const': 'dog', 'enum': ['dog'], 'title': 'Type', 'type': 'string'}},
+                'properties': {'type': {'const': 'dog', 'title': 'Type', 'type': 'string'}},
                 'required': ['type'],
             },
         },
@@ -576,22 +567,34 @@ def test_class_var_as_string(create_module):
         # language=Python
         """
 from __future__ import annotations
-from typing import ClassVar
+from typing import Annotated, ClassVar, ClassVar as CV
 from pydantic import BaseModel
 
 class Model(BaseModel):
     a: ClassVar[int]
     _b: ClassVar[int]
     _c: ClassVar[Forward]
+    _d: Annotated[ClassVar[int], ...]
+    _e: CV[int]
+    _f: Annotated[CV[int], ...]
+    # Doesn't work as of today:
+    # _g: CV[Forward]
 
 Forward = int
 """
     )
 
-    assert module.Model.__class_vars__ == {'a', '_b', '_c'}
+    assert module.Model.__class_vars__ == {'a', '_b', '_c', '_d', '_e', '_f'}
     assert module.Model.__private_attributes__ == {}
 
 
+def test_private_attr_annotation_not_evaluated() -> None:
+    class Model(BaseModel):
+        _a: 'UnknownAnnotation'
+
+    assert '_a' in Model.__private_attributes__
+
+
 def test_json_encoder_str(create_module):
     module = create_module(
         # language=Python
@@ -626,12 +629,6 @@ class Model(BaseModel):
     assert m.model_dump_json() == '{"foo_user":{"x":"user1"},"user":"User(user2)"}'
 
 
-skip_pep585 = pytest.mark.skipif(
-    sys.version_info < (3, 9), reason='PEP585 generics only supported for python 3.9 and above'
-)
-
-
-@skip_pep585
 def test_pep585_self_referencing_generics(create_module):
     module = create_module(
         # language=Python
@@ -655,7 +652,6 @@ class SelfReferencing(BaseModel):
     assert obj.names == [SelfReferencing(names=[])]
 
 
-@skip_pep585
 def test_pep585_recursive_generics(create_module):
     @create_module
     def module():
@@ -722,9 +718,15 @@ def test_recursive_models_union(create_module):
     # This test should pass because PydanticRecursiveRef.__or__ is implemented,
     # not because `eval_type_backport` magically makes `|` work,
     # since it's installed for tests but otherwise optional.
+    # When generic models are involved in recursive models, parametrizing a model
+    # can result in a `PydanticRecursiveRef` instance. This isn't ideal, as in the
+    # example below, this results in the `FieldInfo.annotation` attribute being changed,
+    # e.g. for `bar` to something like `PydanticRecursiveRef(...) | None`.
+    # We currently have a workaround (avoid caching parametrized models where this bad
+    # annotation mutation can happen).
     sys.modules['eval_type_backport'] = None  # type: ignore
     try:
-        module = create_module(
+        create_module(
             # language=Python
             """
 from __future__ import annotations
@@ -745,13 +747,9 @@ class Bar(BaseModel, Generic[T]):
     finally:
         del sys.modules['eval_type_backport']
 
-    assert module.Foo.model_fields['bar'].annotation == typing.Optional[module.Bar[str]]
-    assert module.Foo.model_fields['bar2'].annotation == typing.Union[int, module.Bar[float]]
-    assert module.Bar.model_fields['foo'].annotation == module.Foo
-
 
 def test_recursive_models_union_backport(create_module):
-    module = create_module(
+    create_module(
         # language=Python
         """
 from __future__ import annotations
@@ -773,10 +771,6 @@ class Bar(BaseModel, Generic[T]):
 """
     )
 
-    assert module.Foo.model_fields['bar'].annotation == typing.Optional[module.Bar[str]]
-    assert module.Foo.model_fields['bar2'].annotation == typing.Union[int, str, module.Bar[float]]
-    assert module.Bar.model_fields['foo'].annotation == module.Foo
-
 
 def test_force_rebuild():
     class Foobar(BaseModel):
@@ -850,8 +844,9 @@ def test_nested_more_annotation(create_module):
 def test_nested_annotation_priority(create_module):
     @create_module
     def module():
+        from typing import Annotated
+
         from annotated_types import Gt
-        from typing_extensions import Annotated
 
         from pydantic import BaseModel
 
@@ -1009,11 +1004,11 @@ def test_undefined_types_warning_raised_by_usage(create_module):
 
 
 def test_rebuild_recursive_schema():
-    from typing import ForwardRef, List
+    from typing import ForwardRef
 
     class Expressions_(BaseModel):
         model_config = dict(undefined_types_warning=False)
-        items: List["types['Expression']"]
+        items: list["types['Expression']"]
 
     class Expression_(BaseModel):
         model_config = dict(undefined_types_warning=False)
@@ -1026,7 +1021,7 @@ def test_rebuild_recursive_schema():
 
     class allOfExpressions_(BaseModel):
         model_config = dict(undefined_types_warning=False)
-        items: List["types['Expression']"]
+        items: list["types['Expression']"]
 
     types_namespace = {
         'types': {
@@ -1047,12 +1042,10 @@ def test_forward_ref_in_generic(create_module: Any) -> None:
 
     @create_module
     def module():
-        import typing as tp
-
         from pydantic import BaseModel
 
         class Foo(BaseModel):
-            x: tp.Dict['tp.Type[Bar]', tp.Type['Bar']]
+            x: dict['type[Bar]', type['Bar']]
 
         class Bar(BaseModel):
             pass
@@ -1068,12 +1061,10 @@ def test_forward_ref_in_generic_separate_modules(create_module: Any) -> None:
 
     @create_module
     def module_1():
-        import typing as tp
-
         from pydantic import BaseModel
 
         class Foo(BaseModel):
-            x: tp.Dict['tp.Type[Bar]', tp.Type['Bar']]
+            x: dict['type[Bar]', type['Bar']]
 
     @create_module
     def module_2():
@@ -1105,14 +1096,12 @@ def test_pydantic_extra_forward_ref_separate_module(create_module: Any) -> None:
 
     @create_module
     def module_1():
-        from typing import Dict
-
         from pydantic import BaseModel, ConfigDict
 
         class Bar(BaseModel):
             model_config = ConfigDict(defer_build=True, extra='allow')
 
-            __pydantic_extra__: 'Dict[str, int]'
+            __pydantic_extra__: 'dict[str, int]'
 
     module_2 = create_module(
         f"""
@@ -1130,3 +1119,311 @@ class Foo(BaseModel):
     ]
 
     assert extras_schema == {'type': 'int'}
+
+
+@pytest.mark.xfail(
+    reason='While `get_cls_type_hints` uses the correct module ns for each base, `collect_model_fields` '
+    'will still use the `FieldInfo` instances from each base (see the `parent_fields_lookup` logic). '
+    'This means that `f` is still a forward ref in `Foo.model_fields`, and it gets evaluated in '
+    '`GenerateSchema._model_schema`, where only the module of `Foo` is considered.'
+)
+def test_uses_the_correct_globals_to_resolve_model_forward_refs(create_module):
+    @create_module
+    def module_1():
+        from pydantic import BaseModel
+
+        class Bar(BaseModel):
+            f: 'A'
+
+        A = int
+
+    module_2 = create_module(
+        f"""
+from {module_1.__name__} import Bar
+
+A = str
+
+class Foo(Bar):
+    pass
+        """
+    )
+
+    assert module_2.Foo.model_fields['f'].annotation is int
+
+
+@pytest.mark.xfail(
+    reason='We should keep a reference to the parent frame, not `f_locals`. '
+    "It's probably only reasonable to support this in Python 3.14 with PEP 649."
+)
+def test_can_resolve_forward_refs_in_parent_frame_after_class_definition():
+    def func():
+        class Model(BaseModel):
+            a: 'A'
+
+        class A(BaseModel):
+            pass
+
+        return Model
+
+    Model = func()
+
+    Model.model_rebuild()
+
+
+def test_uses_correct_global_ns_for_type_defined_in_separate_module(create_module):
+    @create_module
+    def module_1():
+        from dataclasses import dataclass
+
+        @dataclass
+        class Bar:
+            f: 'A'
+
+        A = int
+
+    module_2 = create_module(
+        f"""
+from pydantic import BaseModel
+from {module_1.__name__} import Bar
+
+A = str
+
+class Foo(BaseModel):
+    bar: Bar
+        """
+    )
+
+    module_2.Foo(bar={'f': 1})
+
+
+def test_uses_the_local_namespace_when_generating_schema():
+    def func():
+        A = int
+
+        class Model(BaseModel):
+            __pydantic_extra__: 'dict[str, A]'
+
+            model_config = {'defer_build': True, 'extra': 'allow'}
+
+        return Model
+
+    Model = func()
+
+    A = str  # noqa: F841
+
+    Model.model_rebuild()
+    Model(extra_value=1)
+
+
+def test_uses_the_correct_globals_to_resolve_dataclass_forward_refs(create_module):
+    @create_module
+    def module_1():
+        from dataclasses import dataclass
+
+        A = int
+
+        @dataclass
+        class DC1:
+            a: 'A'
+
+    module_2 = create_module(f"""
+from dataclasses import dataclass
+
+from pydantic import BaseModel
+
+from {module_1.__name__} import DC1
+
+A = str
+
+@dataclass
+class DC2(DC1):
+    b: 'A'
+
+class Model(BaseModel):
+    dc: DC2
+    """)
+
+    Model = module_2.Model
+
+    Model(dc=dict(a=1, b='not_an_int'))
+
+
+@pytest.mark.skipif(sys.version_info < (3, 12), reason='Requires PEP 695 syntax')
+def test_class_locals_are_kept_during_schema_generation(create_module):
+    create_module(
+        """
+from pydantic import BaseModel
+
+class Model(BaseModel):
+    type Test = int
+    a: 'Test | Forward'
+
+Forward = str
+
+Model.model_rebuild()
+        """
+    )
+
+
+def test_validate_call_does_not_override_the_global_ns_with_the_local_ns_where_it_is_used(create_module):
+    from pydantic import validate_call
+
+    @create_module
+    def module_1():
+        A = int
+
+        def func(a: 'A'):
+            pass
+
+    def inner():
+        A = str  # noqa: F841
+
+        from module_1 import func
+
+        func_val = validate_call(func)
+
+        func_val(a=1)
+
+
+def test_uses_the_correct_globals_to_resolve_forward_refs_on_serializers(create_module):
+    # Note: unlike `test_uses_the_correct_globals_to_resolve_model_forward_refs`,
+    # we use the globals of the underlying func to resolve the return type.
+    @create_module
+    def module_1():
+        from typing import Annotated
+
+        from pydantic import (
+            BaseModel,
+            PlainSerializer,  # or WrapSerializer
+            field_serializer,  # or model_serializer, computed_field
+        )
+
+        MyStr = str
+
+        def ser_func(value) -> 'MyStr':
+            return str(value)
+
+        class Model(BaseModel):
+            a: int
+            b: Annotated[int, PlainSerializer(ser_func)]
+
+            @field_serializer('a')
+            def ser(self, value) -> 'MyStr':
+                return str(self.a)
+
+    class Sub(module_1.Model):
+        pass
+
+    Sub.model_rebuild()
+
+
+@pytest.mark.xfail(reason='parent namespace is used for every type in `NsResolver`, for backwards compatibility.')
+def test_do_not_use_parent_ns_when_outside_the_function(create_module):
+    @create_module
+    def module_1():
+        import dataclasses
+
+        from pydantic import BaseModel
+
+        @dataclasses.dataclass
+        class A:
+            a: 'Model'  # shouldn't resolve
+            b: 'Test'  # same
+
+        def inner():
+            Test = int  # noqa: F841
+
+            class Model(BaseModel, A):
+                pass
+
+            return Model
+
+        ReturnedModel = inner()  # noqa: F841
+
+    assert module_1.ReturnedModel.__pydantic_complete__ is False
+
+
+# Tests related to forward annotations evaluation coupled with PEP 695 generic syntax:
+
+
+@pytest.mark.skipif(sys.version_info < (3, 12), reason='Test related to PEP 695 syntax.')
+def test_pep695_generics_syntax_base_model(create_module) -> None:
+    mod_1 = create_module(
+        """
+from pydantic import BaseModel
+
+class Model[T](BaseModel):
+    t: 'T'
+        """
+    )
+
+    assert mod_1.Model[int].model_fields['t'].annotation is int
+
+
+@pytest.mark.skipif(sys.version_info < (3, 12), reason='Test related to PEP 695 syntax.')
+def test_pep695_generics_syntax_arbitrary_class(create_module) -> None:
+    mod_1 = create_module(
+        """
+from typing import TypedDict
+
+class TD[T](TypedDict):
+    t: 'T'
+        """
+    )
+
+    with pytest.raises(ValidationError):
+        TypeAdapter(mod_1.TD[str]).validate_python({'t': 1})
+
+
+@pytest.mark.skipif(sys.version_info < (3, 12), reason='Test related to PEP 695 syntax.')
+def test_pep695_generics_class_locals_take_priority(create_module) -> None:
+    # As per https://github.com/python/cpython/pull/120272
+    mod_1 = create_module(
+        """
+from pydantic import BaseModel
+
+class Model[T](BaseModel):
+    type T = int
+    t: 'T'
+        """
+    )
+
+    # 'T' should resolve to the `TypeAliasType` instance, not the type variable:
+    assert mod_1.Model[int].model_fields['t'].annotation.__value__ is int
+
+
+@pytest.mark.skipif(sys.version_info < (3, 12), reason='Test related to PEP 695 syntax.')
+def test_annotation_scope_skipped(create_module) -> None:
+    # Documentation:
+    # https://docs.python.org/3/reference/executionmodel.html#annotation-scopes
+    # https://docs.python.org/3/reference/compound_stmts.html#generic-classes
+    # Under the hood, `parent_frame_namespace` skips the annotation scope so that
+    # we still properly fetch the namespace of `func` containing `Alias`.
+    mod_1 = create_module(
+        """
+from pydantic import BaseModel
+
+def func() -> None:
+    Alias = int
+
+    class Model[T](BaseModel):
+        a: 'Alias'
+
+    return Model
+
+Model = func()
+        """
+    )
+
+    assert mod_1.Model.model_fields['a'].annotation is int
+
+
+@pytest.mark.skipif(
+    platform.python_implementation() == 'PyPy' and sys.version_info < (3, 11),
+    reason='Flaky on PyPy',
+)
+def test_implicit_type_alias_recursive_error_message() -> None:
+    Json = list['Json']
+
+    with pytest.raises(RecursionError, match='.*If you made use of an implicit recursive type alias.*'):
+        TypeAdapter(Json)
